/* AbelCam Scripts (c) seiz.ch 2017 */

var camObjs;				// camera properties for all cameras /config.xml
var activeCamera;			// active camera object
var camNrInObjs = -1		// internal number of camera
var currentCam = -1;		// selected camera
var camInit = false;		// camera display initialized
var sliderInit = false; 	// ptz sliders initialized
var imgSrc = '';			// jpeg image url
var docWidth = 0;			// detect phone rotation
var imgWidth = 0;			
var imgHeight= 0;
var srcWidth = 0;			// original image width
var srcHeight = 0;			// original image height

var imgRequested = 0;		// last request time
var imgInterval = 0;		// interval in milliseconds
var stoppedVideo = false;	//
var vserver = '';			// windows media URL
var clientStats = false;	// client status enabled
var doReplay = false;		// replay available
var havePresets = false;	// presets available
var overRide = false;		// avoid loop when reading PTZ
var isSliding = false;		// detect sliding and don't request position
var u						// unique value to prevent caching
var timers = {};			// for event handling
var isMobile = false;		// running on a mobile device
var handlesMJPEG = true;	// browser can handle motion jpeg
var wantsJPEG = false;		// wants JPEG in all browsers
var forceMJPEG = false;		// don't show on browsers without MJPEG
var startedWMV = false;		// WMV Stream started
var isMSBrowser = false;	// Microsoft Browser
var isIE = false			// Microsoft Internet Explorer
var webRoot = '';			// AbelCam host and port if not on the same location as the scripts
var debug = false;


function doReload() {
		if (debug) {
			console.log('doReload');
		}
		camInit = false;
		sliderInit = false;
		if (window.RT) clearTimeout(window.RT);
		window.RT = setTimeout(function()
		{
			this.location.reload(false); /* false to get page from cache */
		}, 100);
}

function reInit() {
	if (debug) {
		console.log('reInit');
	}
	if (activeCamera) {
		init_video(activeCamera);
		init_sliders(activeCamera);
	} else {
		camObjs = readCams(currentCam);
	}
}

/* ptz ------------------------------------------------------------------------------------- */
function setPtz() {
	// set new PTZ position
	if (! overRide) {
		var thisPTZ = '';
		if (activeCamera.PT == "True") {
			thisPTZ = '/pos' + currentCam + '-' + $( "#pan" ).val() + ',' + $( "#tilt" ).val() + '&zoom' + currentCam + '-' + $( "#zoom" ).val();
		} else {
			thisPTZ = '/zoom' + currentCam + '-' + $( "#zoom" ).val();
		}
		$.get( thisPTZ );
		waitForFinalEvent(function(){
			getPtz();
		}, 1500, "getPTZ");
	}
}

function getPtz() {
	// read PTZ Positions
	var thisPTZ = webRoot + '/getpos' + currentCam + '.xml';
	var ptzPos;
	if (debug) {
		console.log('get PTZ Positions');
	}
	if (! isSliding) {
		$.ajax({ 
			url: thisPTZ,
			dataType: 'text',
			success: function(xml) {
				if (debug) {
					console.log('get PTZ Positions success');
				}
				ptzPos = parseXml(xml);
				overRide = true;
				if (activeCamera.PT == "True") {
					$( "#slider-pan" ).slider('value', parseFloat(ptzPos.xml.pan));
					$( "#slider-tilt" ).slider('value', parseFloat(ptzPos.xml.tilt));
				}
				$( "#slider-zoom" ).slider('value', parseFloat(ptzPos.xml.zoom) * 100);
				overRide = false;
			},
			error: function() {
				if (debug) {
					console.log('get PTZ Positions failure');
				}
			}
		});
	}
	waitForFinalEvent(function(){
		getPtz();
	}, 3000, "getPTZ");
}

/* refresh image --------------------------------------------------------------------------- */

function loadedFirstImage(activeCamera) {
	camInit = true;
	if (debug) { console.log('loadedFirstImage'); }
	if (sliderInit) {
		resizeSliders(activeCamera);
	}
	var fps = parseFloat(activeCamera.Fps); // frames per second
	imgInterval = 1000 / fps;
	
	startVideo();
	
}

function startVideo() {
	if (startedWMV) {
		startedWMV = false;
		reInit();
	}
	if (debug) {
		console.log('startVideo, handlesMJPEG = ' + handlesMJPEG);
	}
	if (handlesMJPEG) {
		// browser can handle motion jpeg
		imgSrc = imgSrc.replace(/\.jpg/,'.mjpg');
		u = new Date();
		u = u.getTime();
		$('#liveimage').attr('src',imgSrc + '?u=' + u ); 
		//$('#liveimage').unbind();
		$('#liveimage').one('load', function( event ) {
			loadedMJPEG();
		});
	} else {
		if (! forceMJPEG) {
			stoppedVideo = false;
			loadImage(100);
		} else {
			if (vserver) {
				// we can't show MJPEG, but you forced me to. 
				// But the Video Server is on, and this is probably a MS browser. 
				// Give it a try:
				startWMV();
			}
		}
	}
	// Start/Stop Button handling
	$('.startVideo').html('Stop');
	$('.startVideo').attr('title','Click here to stop displaying images');
	$('.startVideo').show();
	$('.startVideo').attr('onclick','stopVideo()');
	if (debug) {
		console.log('startVideo done, imgSrc = ' + imgSrc);
	}
}

function restartVideo() {
	if (debug) {
		console.log('restartVideo()');
	}
	imgSrc = imgSrc.replace(/\.mjpg/,'.jpg');
	$('#liveimage').one('load', function( event ) {
		startVideo();
	});
	$('#liveimage').attr('src',imgSrc); 
	//$('#liveimage').unbind();
}

function stopVideo() {
	if (handlesMJPEG) {
		// browser can handle motion jpeg
		imgSrc = imgSrc.replace(/\.mjpg/,'.jpg');
	} else {
		stoppedVideo = true;
	}
	// FireFox needs more to stop a motion jpeg stream
	$('#liveimage').attr('src',"#"); 
	if (imgSrc.indexOf('current') != -1) {
		$('#liveimage').attr('src',imgSrc); 
	} else {
		// need still picture here
		$('#liveimage').attr('src',imgSrc.replace(/replay/,'current'));
	}
	
	// Start/Stop Button handling
	$('.startVideo').html('Start');
	$('.startVideo').attr('title','Click here to start displaying images');
	$('.startVideo').attr('onclick','startVideo()');
	if (debug) {
		console.log('stopVideo done, imgSrc = ' + imgSrc);
	}
}

function startReplay() {
	stopVideo();
	imgSrc = imgSrc.replace(/current/,'replay');
	$('.replay').hide();
	$('.ptzActive').hide();
	$('.camera').show();
	if (havePresets) { $('.preset-area').hide(); }
	startVideo();
	if (debug) {
		console.log('startReplay done, imgSrc = ' + imgSrc);
	}
}

function stopReplay() {
	stopVideo();
	imgSrc = imgSrc.replace(/replay/,'current');
	$('.replay').show();
	$('.ptzActive').show();
	$('.camera').hide();
	if (havePresets) { $('.preset-area').show(); }
	startVideo();
	if (debug) {
		console.log('stopReplay done, imgSrc = ' + imgSrc);
	}
}


function loadedImage() {
	if (debug) { console.log('loadedImage'); }
	// gone one, plan for next one
	// all times im milliseconds
	var now = +new Date();
	var timeToLoad = now - imgRequested; // how long does it take to load an image
	if ( stoppedVideo ) {
		if (debug) { console.log('stopped Video'); }
	} else {
		if (imgRequested + imgInterval - timeToLoad > now) {
			// we're early, wait a bit
			var waitTime = imgRequested + imgInterval - timeToLoad - now;
			if (debug) { console.log('waitTime ' + waitTime); }
			loadImage(waitTime);
			if (debug) { console.log('loadedImage, wait'); }
		} else {
			// get image (almost) immediately
			loadImage(100);
			if (debug) { console.log('loadedImage, nowait'); }
		}
	}
}

function loadImage(waitTime) {
	// get next one
	if (waitTime > 0) {
		waitForFinalEvent(function(){
			imgRequested = +new Date();
			$('#liveimage').attr('src',imgSrc + '?' + imgRequested); // make sure request won't get stuck in any cache
			$('#liveimage').attr('onload','loadedImage()');
		}, waitTime, "getImage");
	} else {
		$('#liveimage').attr('src',imgSrc + '?' + imgRequested);
		$('#liveimage').attr('onload','loadedImage()');
	}
	
	if (debug) {
		console.log('loadImage imgSrc = ' + imgSrc);
	}
	
}

function loadedMJPEG() {
	// this fires way too early - not when the image stream is completed, but then the first image is loaded
	if (debug) {
		console.log('loadedMJPEG()');
	}
	$('#liveimage').one('click', function( event ) {
		restartVideo();
	});
}

function startWMV() {
	if (! startedWMV) {
		stopVideo();
		$('.startVideo').hide();
		var iHtml = '<object classid="clsid:22D6F312-B0F6-11D0-94AB-0080C74C7E95" width="' + imgWidth + '" height="' + imgHeight + '" codebase="http://www.microsoft.com/Windows/MediaPlayer/">';
		iHtml = iHtml + '<param name="Filename" value="' + vserver + '"><param name="AutoStart" value="true">';
		iHtml = iHtml + '<embed src="' + vserver + '" type="application/x-mplayer2" autostart="1" enabled="1" showstatusbar="0" showdisplay="1" ';
		iHtml = iHtml + 'showcontrols="1" pluginspage="http://www.microsoft.com/Windows/MediaPlayer/" CODEBASE="http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=6,0,0,0" ';
		iHtml = iHtml + 'width="' + imgWidth + '" height="' + imgHeight + '"></embed></object>';
		iHtml = '<object id="mediaplayer" classid="clsid:22d6f312-b0f6-11d0-94ab-0080c74c7e95" codebase="http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#version=5,1,52,701" standby="loading microsoft windows media player components..." type="application/x-oleobject" width="' + imgWidth + '" height="' + imgHeight + '">';
		iHtml = iHtml + '<param name="filename" value="' + vserver + '">';
		iHtml = iHtml + '<param name="animationatstart" value="true">';
		iHtml = iHtml + '<param name="transparentatstart" value="true">';
		iHtml = iHtml + '<param name="autostart" value="true">';
		iHtml = iHtml + '<param name="showcontrols" value="false">';
		iHtml = iHtml + '<param name="ShowStatusBar" value="false">';
		iHtml = iHtml + '<param name="windowlessvideo" value="true">';
		iHtml = iHtml + '<embed src="' + vserver + '" autostart="true" showcontrols="false" showstatusbar="0" bgcolor="white" width="' + imgWidth + '" height="' + imgHeight + '">';
		iHtml = iHtml + '</object>';
		$('#showHere').html(iHtml);
		$('.wmvbutton').html('Stop WMV');
		startedWMV = true;
	} else {
		startedWMV = false;
		reInit();
	}
}


/* init ------------------------------------------------------------------------------------- */	
function readCams(currentCam){ 
	// read cam properties from /config.xml
	// initialize all display elements
	camObjs = null;
	activeCamera = null;
	var config = webRoot + "/config.xml";
	if (debug) {
			console.log('.ajax');
	}
	$.ajax({ 
		url: config,
		dataType: 'text',
		success: function(xml) {
			if (debug) {
				console.log('ajax success');
			}
			camObjs = parseXml(xml);
			var iHtml = ''
			var camNr;
			if (camObjs.Config.Camera != undefined) {
				if (camObjs.Config.Camera.length != undefined) {
					// multiple cameras
					for (var c = 0; c < camObjs.Config.Camera.length; c++) {
						iHtml = handleCameraEntry(camObjs.Config.Camera[c], iHtml, c)
					}
				} else {
					// only one active
					iHtml = handleCameraEntry(camObjs.Config.Camera, iHtml, 0)
				}
			} else {
				if ($('.nocam').length == 0) {
					$('.EdgeIE').prepend('<p class="nocam">No Camera active.</p>');
					$('.initDisplay').hide();
				}
			}
			if (clientStats) {
				iHtml = iHtml + '<li '; 
				if (location.href.indexOf('clientstats.html') > -1) {
					iHtml = iHtml + ' class="active"';
				}
				iHtml = iHtml + '><a href="clientstats.html" title="Access Statistics">Stats</a></li>';
			}
			$('.nav').html(iHtml);
			
			if (camNrInObjs > -1) {
				if (debug) {
					console.log('init calls');
				}
				init_sliders(activeCamera);
				init_video(activeCamera);
				if (vserver > '') { init_vserver(); }
				if (doReplay) { init_replay(); }
				if (havePresets) { init_presets(); }
				if (isIE) { init_silver(); }
			}
			docWidth = $( document ).width();
			$('.initDisplay').attr('onclick','reInit()');
			$(window).resize(function () {
				if (docWidth != $( document ).width()) {
					waitForFinalEvent(function(){
						//doReload();
						reInit();
					}, 500, "window resize");
				} else {
					// document height changes on chrome with the address bar popping in and out
				}
			});
			plug();
		},
		error: function() {
			if ($('.ajaxerror').length == 0) {
				var iHtml = '<p class="ajaxerror">Error reading ' + url +'.</p>';
				$('.EdgeIE').prepend(iHtml);
			}
		}
	});
};

function handleCameraEntry(Camera, iHtml, c) {
	iHtml = iHtml + '<li' ;
	camNr = parseInt(Camera.Number);
	if (camNr == currentCam) {
		if (location.href.indexOf('stat') == -1) {
			iHtml = iHtml + ' class="active"';
		}
		camNrInObjs = c;
		activeCamera = Camera;
		
	}
	iHtml = iHtml + '><a href="index.html?cam=' + camNr + '&u=' + u + '"';
	if (Camera.Description) {
			iHtml = iHtml +  ' title="' + Camera.Description + '"';
	}
	iHtml = iHtml + '>' + Camera.Name + '</li>';
	return iHtml;
}
	
	
function init_video(camObjs,camNr) {
	if (debug) {
		console.log('init_video');
	}
	var vwidth = $('#cam').width();
	$('.title').text(activeCamera.Name);
	if (activeCamera.DescriptionHTML) {
		$('#description').html(activeCamera.DescriptionHTML);
	} else {
		$('#description').text(activeCamera.Description);
	}
	if (srcWidth > 0 && srcHeight > 0) {
		// known cam resolution from includes
		var vheight = Math.round(vwidth / srcWidth * srcHeight);
		imgSrc = webRoot + '/current' + activeCamera.Number + '.jpg?size=' + vwidth + 'x' + vheight;
	} else {
		imgSrc = webRoot + '/current' + activeCamera.Number + '.jpg?width=' + vwidth;
	}
		
	
	imgWidth = vwidth;
	$('#showHere').html('<img id="liveimage" src="' + imgSrc + '" width="' + vwidth + '">'); 
	$('#liveimage').one('load', loadedFirstImage(activeCamera));

	if (! handlesMJPEG && forceMJPEG) {
		// browser can't you want to force it -> message
		if ($('.blink').length == 0) {
			if (! vserver) {
				$('.EdgeIE').append('<p class="blink">Unless you change your browser, you will not see a video stream - only this single image.</p>');
			} else {
				$('.EdgeIE').append('<p class="blink">While this page might be showing a wmv video stream, this page works better in a different browser.</p>');
			}
		}
	}
}
	
function init_sliders(activeCamera) {
	sliderInit = true;
	if (debug) {
		console.log('init_sliders');
	}
	var dwidth = $( document ).width();
	var pzorientation = 'vertical';
	var torientation = 'horizontal';
	if (dwidth < 992) {
		pzorientation = 'horizontal';
		torientation = pzorientation;
	}
	if (activeCamera.PT == "True") {
		init_slider("pan", torientation, parseFloat(activeCamera.Pan),-100,100,1);	
		$('#slider-pan').width($('.pan').width());
		$('.pan').show();
		init_slider("tilt", pzorientation, parseFloat(activeCamera.Tilt),-100,100,1);
		$('.tilt').show();
		$('.panhandle').show();
	} else {
		$('.panhandle').hide();
		$('.tilthandle').hide();
		$('.pan').hide();
		$('.tilt').hide();
	}
	if (activeCamera.ZoomMin != activeCamera.ZoomMax) {
		init_slider("zoom", pzorientation, parseFloat(activeCamera.Zoom) * 100,
			parseFloat(activeCamera.ZoomMin) * 100,
			parseFloat(activeCamera.ZoomMax) * 100, 0.01);
		$( "#zoom").val( $( "#slider-zoom").slider( "value" ) / 100 );
		$('.zoom').show();
		$('.zoomhandle').show();
	} else {
		$('.zoom').hide();
		$('.zoomhandle').hide();
	}
}
	
function init_slider(s, o, v,mi,ma, multiplikator) {
	if (debug) {
		console.log('init_slider ' + s);
	}
	$( "#slider-" + s ).slider({
		orientation: o,
		range: "min",
		min: mi,
		max: ma,			
		value: v,
		slide: function( event, ui ) {
			$( "#" + s ).val( Math.round(ui.value * multiplikator * 100) / 100);
			isSliding = true;
		},
		stop: function (event, ui) {
			isSliding = false;
			setPtz();
		}
	});
	// set initial value
	$( "#" + s ).val( $( "#slider-" + s ).slider( "value" ) );
	// set class to toggle visibility
	$( "#slider-" + s ).addClass('ptzActive');
	if (o == 'horizontal') {
		$("#slider-" + s).css('cursor', 'w-resize;');
	} else {
		$("#slider-" + s).css('cursor', 'n-resize;');
	}
}

	
	
function resizeSliders(activeCamera) {
	if (debug) {
		console.log('resizeSliders');
	}
	
	var dwidth = $( document ).width();
	// resize zoom and tilt sliders
	var defSize = 8;
	imgWidth = $('#liveimage').width();
	imgHeight = $('#liveimage').height();
	
	// cam info
	if (currentCam > -1) {
		if (imgHeight == 0 || imgHeight > imgWidth) {
			// imgHeight not known or unlikely
			imgHeight = Math.round(imgWidth / srcWidth * srcHeight);
		}
	} else {
		if (imgHeight == 0) {
			// height not known yet, try later
			window.setTimeout(resizeSliders(activeCamera), 500);
		}
	}
	


	$('.initDisplay').hide();
	if (activeCamera.PT == "True") {
		if (dwidth >=992) {
			// vertical tilt
			$('#slider-tilt').height(imgHeight);
			$('#slider-tilt').width(defSize);
			// horizontal pan
			$('#slider-pan').height(defSize);
			$('#slider-pan').width(imgWidth);
		} else {
			// horizontal tilt
			$('#slider-tilt').height(defSize);
			$('#slider-tilt').width(imgWidth);
			// horizontal pan
			$('#slider-pan').height(defSize);
			$('#slider-pan').width(imgWidth);
		}
	}
	if (activeCamera.ZoomMin != activeCamera.ZoomMax) {
		if (dwidth >=992) {
			// vertical zoom
			$('#slider-zoom').height(imgHeight);
			$('#slider-zoom').width(defSize);
		} else {
			// horizontal zoom
			$('#slider-zoom').height(defSize);
			$('#slider-zoom').width(imgWidth);
		}
	}
}
	
function init_vserver() {
	// set vserver url
	if (debug) {
		console.log('init_vserver');
	}
	if (vserver.indexOf('current' + currentCam) != -1) {
		if (isMSBrowser) {
			WMVButton(); 	// Play in page
		} else {
			WMVUrl();		// external link
		}
	} else {
		// video server url does not match current camera number - there's a bug somewhere
	}
}

function WMVUrl() {
	var	iHtml = iHtml = '<a href="' + vserver + '&u=' + u + '" target="_blank" on>Windows Media Stream</a>';
	$('.wmv').html(iHtml);
	$('.wmv').show();
}
function WMVButton() {
	var iHtml = '<button class="wmvbutton" title="Click here to see the Windows Media Stream">WMV</button>';
	$('.wmv').html(iHtml);
	$('.wmv').show();
	$('.wmv').attr('onclick','startWMV()');
}

function init_replay() {
	// set replay button
	if (debug) {
		console.log('init_replay');
	}
	if ($('.replay').length == 0) {
		var iHtml = '<button class="replay" title="Click here to see recorded images">Replay</button><button class="camera" title="Click here to see live images">Camera</button>';
		$('.replay-area').append(iHtml);
	}
	$('.replay').show();
	$('.ptzActive').show();
	$('.camera').hide();
	$('.replay').attr('onclick','startReplay()');
	$('.camera').attr('onclick','stopReplay()');
}

function init_presets() {
	var presetCounter = 0;
	var presets = "Preset ";
	// add buttons
	while ( presetCounter < presetNameArray.length ){
		presets = presets + '<button class="preset preset' + presetCounter + '" title="Click here to move to preset position">' + presetNameArray[presetCounter] + '</button>';
		presetCounter++;
	}
	$('.preset-area').html(presets);
	
	// add button events
	presetCounter = 0;
	while ( presetCounter < presetNameArray.length ){
		$('.preset' + presetCounter).attr('onclick','setPreset('+presetPanArray[presetCounter]+','+presetTiltArray[presetCounter]+','+presetZoomArray[presetCounter]+')');
		presetCounter++;
	}
	if (presetCounter == 0) {
		havePresets = false;
		$('.preset-area').hide();
	}
}
function setPreset(p, t, z) {
	isSliding = true;
	$( "#slider-pan").slider('value', p);
	$( "#pan").val(p);
	$( "#slider-tilt").slider('value', t);
	$( "#tilt").val(t);
	$( "#slider-zoom").slider('value', z * 100);
	$( "#zoom").val(z);
	isSliding = false;
	setPtz();
}

function init_silver() {
	// goodie for Microsoft Internet Explorer
	if ($('.silver').length == 0) {
		$('.EdgeIE').append('<h2 class="silver"><a href="silver/full.html">Start Silverlight Page</a></h2>');
	}
}

function plug() {
	// shameless plug
	if ($('.plug').length == 0) {
		var iHtml = '<hr><span class="plug"><a href="https://abelcam.com" target="_blank" title="AbelCam"><img class="icon" src="/img/icon.gif" alt="AbelCam Logo"></a>&nbsp;AbelCam<sup>&reg;</sup> by Seiz&nbsp;System&nbsp;Engineering&nbsp;<a href="https://seiz.ch" target="_blank" title="Seiz System Engineering Ltd.">'
		iHtml = iHtml + '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" class="logo" height="48px" width="33px" viewBox="0 0 280 400"><path class="black" d="M146.97 124.4c14.72-24.46 73.75-30.2 85.5 5.3 11.73 35.4-125.9 75.3-155.03 88.8-29.12 13.5-30.16 18.8-28.34 23.2 1.8 4.4 13.5 14.7 36.03 14.3 22.5-.5 57.65-11.2 69.12-18 11.47-6.8 8.32-19.3 1.5-30.4-6.8-11.1-19.28-24.2-30.7-34.6-23.5-18.3-36.73-27.2-59.75-44.9-28.5-21.7-59.82-60.7-38.98-82.2 20.83-21.5 56.56-28.1 82.9-21.8 26.36 6.3 44.8 38.6 38 56-6.78 17.5-34.1 29.6-42.27 25.5-8.18-4.2-8.5-6.4 1.94-9.9 10.4-3.5 32.2-13.3 28.2-32.6C131 44 86.6 27.5 55 44.9c-31.62 17.4-30.82 35 40.84 87.46 14.94 10.7 29.73 21 43.15 32.73 13.4 11.7 23.8 26 33.9 41.6 6.6 10.5 6.2 33.5-16.4 46.4-16.3 9.3-35.1 13.6-52 16.3-40 2.9-61.9-10.1-67.6-23.9-6.2-14.7-1.6-24 32.3-40.8 33.8-16.77 83.7-31 118.1-47.7 4.5-2.4 12.3-6.8 19.8-11.5 8.3-4.8 9.8-15.66 2.4-20.36-23.1-14.54-53.3 20.8-49 32.93 4.2 12.1 7.8 15.2 18.7 29.5 10.9 14.2 35.9 46.5 56.7 77.7 20.8 31.1 28.2 61.3 19.8 78.9-8.4 17.64-41.7 33.6-62.9 34.65-21.3 1.06-54.6 1.34-78.5-21.47-23.9-22.8-18-71.3-4.4-40.3 6.5 15.06 17 33.33 37.8 41.8 20.7 8.42 66 4.5 81.9-11.68 15.9-16.16 15.8-33.7 1.7-58-21.8-31.7-46.7-69.15-56.3-81.97-9.6-12.82-22.3-27.2-29.26-42-4.66-15-7.07-27.03 1.22-40.8z" stroke="#000" stroke-width=".37"></path><path class="animate" id="a" d="M0 0c.63 85.97-102.13-41.02-86.17-59.55 15.96-18.53 71.45-28.7 83.2 6.78 11.73 35.4-125.9 75.3-155.03 88.8-29.12 13.5-30.16 18.8-28.34 23.2 1.8 4.4 13.5 14.73 36.03 14.3 22.5-.5 57.6-11.2 69.1-18 11.4-6.8 8.3-19.28 1.5-30.4-6.8-11.1-19.3-24.2-30.7-34.6-23.5-18.3-36.8-27.15-59.8-44.85-28.5-21.74-59.8-60.74-39-82.24 20.83-21.5 56.56-28.1 82.9-21.8 26.36 6.34 44.8 38.6 38 56.08-6.78 17.5-34.1 29.62-42.27 25.5-8.18-4.15-8.5-6.4 1.94-9.85 10.4-3.5 32.2-13.28 28.2-32.6-4.1-19.2-48.5-35.73-80.1-18.3-31.7 17.4-30.9 35 40.8 87.45 14.9 10.7 29.7 21 43.1 32.76C-83.1-5.56-72.7 8.7-62.6 24.3-56 34.86-56.4 57.8-79 70.76c-16.3 9.3-35.1 13.6-52 16.3-40 2.9-61.9-10.12-67.6-23.9-6.2-14.72-1.6-24.02 32.3-40.8 33.88-16.75 83.7-31 118.16-47.7 4.5-2.4 12.3-6.8 19.8-11.5 8.32-4.8 9.8-15.64 2.4-20.34-23.07-14.53-53.3 20.8-49 32.94C-70.7-12.12-67.1-9.02-56.2 5.26-45.3 19.5-20.3 51.76.5 82.96c20.8 31.16 28.2 61.3 19.8 78.96-8.4 17.67-41.7 33.62-62.9 34.68-21.3 1.07-54.6 1.35-78.5-21.46-23.9-22.8-18-71.3-4.4-40.3 6.5 15.06 17 33.33 37.8 41.77 20.7 8.5 66 4.5 81.9-11.6 15.9-16.1 15.8-33.7 1.7-58-21.8-31.7-46.7-69.1-56.3-81.9C-69.9 12.2-82.5-2.2-89.5-17c-3.43-7.4-9.7-27.3 1.4-40.9C-65.4-81.07-.9-86-.14 0z" fill="none"></path><circle class="reddot" r="22" cy="182" cx="236" fill="red"><animateMotion begin="2s" dur="25s" repeatCount="99" calcMode="paced" additive="sum"><mpath xlink:href="#a"></mpath></animateMotion></circle></svg>';
		iHtml = iHtml + '</a></span><hr>';
		$('.EdgeIE').append(iHtml);
	}
}

/* library functions ------------------------------------------------------------------------ */
function parseXml(xml, arrayTags) {
    var dom = null;
    if (window.DOMParser)     {
        dom = (new DOMParser()).parseFromString(xml, "text/xml");
    } else if (window.ActiveXObject) {
        dom = new ActiveXObject('Microsoft.XMLDOM');
        dom.async = false;
        if (!dom.loadXML(xml)) {
            throw dom.parseError.reason + " " + dom.parseError.srcText;
        }
    } else {
        throw "cannot parse xml string!";
    }

    function isArray(o) {
        return Object.prototype.toString.apply(o) === '[object Array]';
	}

	function parseNode(xmlNode, result) {
        var jsonNode = {};
        var existing = result[xmlNode.nodeName];
        if(existing) {
            if(!isArray(existing)) {
                result[xmlNode.nodeName] = [existing, jsonNode];
            } else {
                result[xmlNode.nodeName].push(jsonNode);
            }
        } else {
            if(arrayTags && arrayTags.indexOf(xmlNode.nodeName) != -1) {
                result[xmlNode.nodeName] = [jsonNode];
            } else {
                result[xmlNode.nodeName] = jsonNode;
            }
        }
		
        var length = xmlNode.childNodes.length;
		if (length == 1) {
			result[xmlNode.nodeName] = xmlNode.childNodes[0].data;
		} else {
			for(var i = 0; i < length; i++) {
				parseNode(xmlNode.childNodes[i], jsonNode);
			}
        }
    }

    var result = {};
    if(dom.childNodes.length) {
        parseNode(dom.childNodes[0], result);
    }

    return result;
}

function waitForFinalEvent(callback, ms, uniqueId) {
    if (!uniqueId) {
      uniqueId = "Don't call this twice without a uniqueId";
    }
	if (debug) { console.log('waitFor uniqueId ' + uniqueId); }
    if (timers[uniqueId]) {
      clearTimeout (timers[uniqueId]);
    }
    timers[uniqueId] = setTimeout(callback, ms);
};
